{%- if pagename != "search" and builder != "singlehtml" %}
<div id="searchbox" style="display: none" role="search">
    <h3>{{ _('Quick Search') }}</h3>
    <div><input type="text" name="q"/></div>
</div>
<link rel="stylesheet" href="https://cdn.jsdelivr.net/docsearch.js/2/docsearch.min.css" />
<script type="text/javascript" src="https://cdn.jsdelivr.net/docsearch.js/2/docsearch.min.js"></script>
<script type="text/javascript">
    $('#searchbox').show(0);
    docsearch({
        apiKey: '4332f80999581036b1bf34e7aa9a2940',
        indexName: 'coala',
        inputSelector: '#searchbox > div > input',
        debug: false // Set debug to true if you want to inspect the dropdown
        });
</script>

{%- endif %}
